//
//  NzAutoTrackAPI.m
//  NzAnalyticsDemo
//
//  Created by FY on 2023/4/12.
//  Copyright © 2023 FY. All rights reserved.
//

#import "NzAutoTrackAPI.h"
#import "UIViewController+NzAutoTrack.h"
#import "NzAnalyticsSDK.h"
#import "NzFileStorage.h"
#import "NzEvent.h"


NSString * const kFileStorageAutoTrackContext = @"io.naza.analytics.storage.api.context";   // 兼容1.0的context
static NzAutoTrackAPI * sharedInstance = nil;

@interface NzAutoTrackAPI ()

@property(nonatomic, strong) NSString * firstLaunch;
@property(nonatomic, strong) NSString * firstLaunchToday;
@property(nonatomic, strong) NSDate * launchTime;
@property(nonatomic, strong) NzEvent * currentPageEvent;
@property(nonatomic, assign) NSTimeInterval currentPageEnterTime;

@property(nonatomic, strong) NzAnalyticsOption * sdkOption;

@end

@implementation NzAutoTrackAPI
+(NzAutoTrackAPI *) shared
{
    if (sharedInstance != nil) {
        return sharedInstance;
    }
    sharedInstance = [[NzAutoTrackAPI alloc] init];
    return sharedInstance;
}

- (instancetype)init
{
    if (self = [super init]) {
        self.sdkOption = [[NzAnalyticsOption alloc] init];
        self.sdkOption.autoTrackAppLaunch = false;
        self.sdkOption.autoTrackAppShow = false;
        self.sdkOption.autoTrackAppHide = false;
        self.sdkOption.autoTrackViewScreenShow = false;
        self.sdkOption.autoTrackViewScreenHide = false;
        self.currentPageEnterTime = 0;
        NSNotificationCenter * nc = [NSNotificationCenter defaultCenter];
        for (NSString *name in @[ UIApplicationDidEnterBackgroundNotification,
                                  UIApplicationDidFinishLaunchingNotification,
                                  UIApplicationWillEnterForegroundNotification,
                                  UIApplicationWillTerminateNotification,
                                  UIApplicationWillResignActiveNotification,
                                  UIApplicationDidBecomeActiveNotification ]) {
            [nc addObserver:self selector:@selector(handleAppStateNotification:) name:name object:nil];
        }
        
        [UIViewController nz_swizzleViewDidAppear];
        //        [UIViewController nz_swizzleViewWillDisappear];
    }
    return self;
}

- (void) pageShowBy:(NSString *)className withContent:(NSString *)content
{
    [self pageHide];
    NzEvent *event = [[NzEvent alloc] init];
    event.event = @"UDC__APP_PAGE_SHOW";
    [event setPropValue:className forKey:@"s_pageName"];
    [event setPropValue:content forKey:@"s_title"];
    [event setPropValue:[self firstLaunchToday] forKey:@"s_isFirstDayVisit"];
    self.currentPageEvent = event;
    self.currentPageEnterTime = [[NSDate date] timeIntervalSince1970];
    if (self.sdkOption.autoTrackViewScreenShow == YES) {
        [[NzAnalyticsSDK shared] track:event];
    }
}

- (void) pageHide
{
    if (self.currentPageEvent == nil || self.currentPageEnterTime == 0) {
        return;
    }
    if (self.sdkOption.autoTrackViewScreenHide == YES) {
        NSTimeInterval duration = [[NSDate date] timeIntervalSince1970] - self.currentPageEnterTime;
        NzEvent* event = [[NzEvent alloc] initWithDictionary:[self.currentPageEvent toDictionary]];
        event.event = @"UDC__APP_PAGE_HIDE";
        [event setPropValue:[NSNumber numberWithInteger:(int)duration] forKey:@"s_pageStayTime"];
        [[NzAnalyticsSDK shared] track:event];
    }
    self.currentPageEnterTime = 0;
}

#pragma mark - private
- (void)handleAppStateNotification: (NSNotification *)notificaiton
{
    NSLog(@"NzAutoTrackAPI >>> handleAppStateNotification name %@", notificaiton.name);
    if ([notificaiton.name isEqualToString:UIApplicationDidFinishLaunchingNotification]) {
        [self enterAppEvent:NO];
    } else if ([notificaiton.name isEqualToString:UIApplicationWillEnterForegroundNotification]) {
        [self enterAppEvent:YES];
    } else if ([notificaiton.name isEqualToString:UIApplicationWillTerminateNotification] ||
               [notificaiton.name isEqualToString:UIApplicationDidEnterBackgroundNotification]) {
        [self exitAppEvent];
    }
}

- (void)enterAppEvent:(BOOL)fromBackground
{
    [self loadContext];
    if ([self.firstLaunch isEqualToString:@"YES"] &&
        self.sdkOption.autoTrackAppLaunch == YES) {
        NzEvent * event = [[NzEvent alloc] init];
        event.event = @"UDC__APP_ACTIVATE";
        [[NzAnalyticsSDK shared] track:event];
    }
    
    if (self.sdkOption.autoTrackAppShow == YES) {
        NzEvent * event = [[NzEvent alloc] init];
        event.event = @"UDC__APP_SHOW";
        //        attr1    是否从后台唤醒    s_isFromBackground
        //        attr2    是否首日访问    s_isFirstDayVisit
        //        attr3    是否首次访问    s_isFirstVisit
        //        attr4    页面名称    s_pageName
        //        attr5    标题    s_title
        [event setPropValue:fromBackground?@"YES":@"NO" forKey:(NSString *)@"s_isFromBackground"];
        [event setPropValue:self.firstLaunchToday forKey:(NSString *)@"s_isFirstDayVisit"];
        [event setPropValue:self.firstLaunch forKey:(NSString *)@"s_isFirstVisit"];
        [[NzAnalyticsSDK shared] track:event];
    }
    if (self.currentPageEvent) {
        [self pageShowBy:(NSString *)[self.currentPageEvent propValueForKey:@"s_pageName"]
             withContent:(NSString *)[self.currentPageEvent propValueForKey:@"s_title"]];
    }
}

- (void)exitAppEvent
{
    [self pageHide];
    if (self.sdkOption.autoTrackAppHide == YES) {
        NzEvent * event = [[NzEvent alloc] init];
        event.event = @"UDC__APP_HIDE";
        //        attr1    是否首日访问    s_isFirstDayVisit
        //        attr2    页面名称    s_pageName
        //        attr3    标题    s_title
        //        attr4    停留时长    s_stayTime
        NSTimeInterval duration = [[NSDate date] timeIntervalSinceDate:self.launchTime];
        [event setPropValue:self.firstLaunchToday forKey:(NSString *)@"s_isFirstDayVisit"];
        [event setPropValue:[NSNumber numberWithInt:(int)duration] forKey:(NSString *)@"s_stayTime"];
        [[NzAnalyticsSDK shared] track:event];
    }
}

- (void)loadContext
{
    NSDictionary * context = [[NzFileStorage sharedInstance] dictionaryForKey:kFileStorageAutoTrackContext];
    if (context == nil) {
        context = [NSDictionary dictionary];
    }
    
    NSDateFormatter *dateFormatter=[[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"yyyy-MM-dd"];
    
    NSNumber * lastLaunchTime = [context objectForKey:@"launchTime"];
    self.launchTime = [NSDate date];
    if(lastLaunchTime != nil) {
        self.firstLaunch = @"NO";
        if ([[dateFormatter stringFromDate:self.launchTime] isEqualToString:[dateFormatter stringFromDate:[NSDate dateWithTimeIntervalSince1970:[lastLaunchTime doubleValue]]]]) {
            self.firstLaunchToday = @"NO";
        } else {
            self.firstLaunchToday = @"YES";
        }
    } else {
        self.firstLaunch = @"YES";
        self.firstLaunchToday = @"YES";
    }
    
    [[NzFileStorage sharedInstance] setDictionary:[NSDictionary dictionaryWithObjectsAndKeys:
                                                  [NSNumber numberWithDouble:[self.launchTime timeIntervalSince1970]] , @"launchTime",
                                                   nil]
                                           forKey:kFileStorageAutoTrackContext];
}

@end
