//
//  UIViewController+NzAutoTrack.m
//  NzAnalytics
//
//  Created by FY on 2018/10/12.
//  Copyright © 2018 FY. All rights reserved.
//

#import <objc/runtime.h>
#import "UIViewController+NzAutoTrack.h"
#import "NzSwizzle.h"
#import "NzUtils.h"
#import "NzAutoTrackAPI.h"

@implementation UIViewController (NzAutoTrack)

- (NSString *)nzAutoTrackContent {
    return objc_getAssociatedObject(self, @"nzAutoTrackContent");
}

- (void)setNzAutoTrackContent:(NSString *)nzAutoTrackContent {
    objc_setAssociatedObject(self, @"nzAutoTrackContent", nzAutoTrackContent, OBJC_ASSOCIATION_COPY);
}

- (BOOL)nzAutoTrackIgnore {
    NSNumber * boolNumber =objc_getAssociatedObject(self, @"nzAutoTrackIgnore");
    if (boolNumber != nil) {
        return [boolNumber boolValue];
    } else {
        return NO;
    }
}

- (void)setNzAutoTrackIgnore:(BOOL)nzAutoTrackIgnore {
    objc_setAssociatedObject(self, @"nzAutoTrackIgnore", [NSNumber numberWithBool:nzAutoTrackIgnore], OBJC_ASSOCIATION_ASSIGN);
}

+ (void)nz_swizzleViewDidAppear
{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        [self nz_replaceMethod:@selector(viewDidAppear:) withMethod:@selector(nz_viewDidAppear:)];
    });
}

+ (void)nz_swizzleViewWillDisappear
{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        [self nz_replaceMethod:@selector(viewWillDisappear:) withMethod:@selector(nz_viewWillDisappear:)];
    });
}

- (void)nz_viewDidAppear:(BOOL)animated
{
    if (self.nzAutoTrackIgnore != YES) {
        UIViewController * currentVC = [NzUtils topViewController];
        if (!currentVC) {
            NzLog(@"Cannot find current UIViewController");
            return;
        }
        
        if (self != currentVC) {
            NzLog(@"I am not currentVC, event show should be reported by currentVC itself");
            return;
        }
//        NzEvent *event = [[NzEvent alloc] init];
//        event.event = @"UDC__APP_PAGE_SHOW";
//        [event setPropValue:NSStringFromClass([currentVC class]) forKey:@"s_pageName"];
//        [event setPropValue:[currentVC nz_elementContent] forKey:@"s_title"];
//        [[NzAnalyticsSDK shared] track:event];
        [[NzAutoTrackAPI shared] pageShowBy:NSStringFromClass([currentVC class]) withContent:[currentVC nz_elementContent]];
    }
    
    [self nz_viewDidAppear:animated];
}

- (void)nz_viewWillDisappear:(BOOL)animated
{
    [self nz_viewWillDisappear:animated];
}

- (NSString *)nz_elementContent
{
    NSString * content;
    if ([self.view conformsToProtocol:@protocol(NzAnalyticsViewAutoTrackProtocol)]
        && [self.view respondsToSelector:@selector(nz_elementContent)]){
        content = [self.view performSelector:@selector(nz_elementContent)];
    }
    if (!content) {
        content = [self title];
    }
    if (!content && self.navigationItem) {
        content = [self.navigationItem title];
    }
    return content;
}

@end
