//
//  NzUtils.m
//  NzAnalyticsDemo
//
//  Created by FY on 2022/5/11.
//  Copyright © 2022 FY. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "NzUtils.h"
#import <UIKit/UIKit.h>

BOOL kLoggerShowLogs = YES;

void nz_dispatch_async(dispatch_queue_t queue, dispatch_block_t block)
{
    dispatch_async(queue, block);
}

void NzLogShow(BOOL showLog)
{
    kLoggerShowLogs = YES;
}

void NzLog(NSString *format, ...)
{
    if (!kLoggerShowLogs) {
        return;
    }
    
    va_list args;
    va_start(args, format);
    NSLogv(format, args);
    va_end(args);
}

id ObjectOrNull(id obj)
{
    return obj ? obj: [NSNull null];
}


@implementation NzUtils

+ (UIViewController *)topViewController
{
    
    UIViewController *rootViewController = [UIApplication sharedApplication].keyWindow.rootViewController;
    UIViewController *currentVC = [NzUtils topViewController:rootViewController];
    return currentVC;
}

+ (UIViewController *)topViewController:(UIViewController *)rootVC
{
    UIViewController *currentVC;
    
    if ([rootVC presentedViewController]) {
        // 视图是被presented出来的
        rootVC = [rootVC presentedViewController];
    }
    
    if ([rootVC isKindOfClass:[UITabBarController class]]) {
        // 根视图为UITabBarController
        currentVC = [self topViewController:[(UITabBarController *)rootVC selectedViewController]];
    } else if ([rootVC isKindOfClass:[UINavigationController class]]){
        // 根视图为UINavigationController
        currentVC = [self topViewController:[(UINavigationController *)rootVC visibleViewController]];
    } else {
        // 根视图为非导航类
        currentVC = rootVC;
    }
    
    return currentVC;
}

+ (NSNumber *)now {
    return [NSNumber numberWithDouble:[[NSDate date] timeIntervalSince1970] * 1000];
}

+ (NSString *)UUID {
    return [[[NSUUID UUID] UUIDString] lowercaseString];
}

+ (NSMutableDictionary *)buildMessageWithDomain:(NSString *)domain action:(NSString *)action data:(NSDictionary *)data {
    return [NSMutableDictionary dictionaryWithObjects:[NSArray arrayWithObjects:
                                                       [NzUtils UUID],
                                                       domain,
                                                       action,
                                                       [NzUtils now],
                                                       data,
                                                       nil]
                                              forKeys:[NSArray arrayWithObjects:
                                                       @"message_id",
                                                       @"domain",
                                                       @"action",
                                                       @"ts",
                                                       @"data",
                                                       nil]];
}

@end
